"""
MVSJ builder functions for MVS Story Generator.
"""
import json
import os
from datetime import datetime
from typing import Dict, Any, Optional, List, Union
from .validation import _validate_and_normalize
from .constants import CAMERA_ANGLES, DEFAULT_COLORS


def _build_mvsj_base(title: Optional[str] = None) -> Dict[str, Any]:
    """Build base MVSJ structure."""
    return {
        "metadata": {
            "version": "1",
            "timestamp": datetime.utcnow().isoformat() + "Z",
            **({"title": title} if title else {})
        },
        "root": {"kind": "root", "children": []}
    }


def _build_multistate_base(title: Optional[str] = None) -> Dict[str, Any]:
    """Build base multi-state MVSJ structure."""
    return {
        "kind": "multiple",
        "metadata": {
            "version": "1", 
            "timestamp": datetime.utcnow().isoformat() + "Z",
            **({"title": title} if title else {})
        },
        "snapshots": []
    }


def _build_structure_chain(url: str, format_type: str = "bcif") -> Dict[str, Any]:
    """Build the common download->parse->structure chain."""
    return {
        "kind": "download",
        "params": {"url": url},
        "children": [{
            "kind": "parse", 
            "params": {"format": format_type},
            "children": [{
                "kind": "structure",
                "params": {"type": "model"},
                "children": []
            }]
        }]
    }


def _build_node(kind: str, params: Optional[Dict] = None, children: Optional[List] = None) -> Dict[str, Any]:
    """Build any MVSJ node with optional params and children."""
    node = {"kind": kind}
    if params:
        node["params"] = params
    if children:
        node["children"] = children
    return node


def _build_component(selector: Union[str, Dict], children: List[Dict]) -> Dict[str, Any]:
    """Build component node with validated selector and children."""
    validated_selector = _validate_and_normalize(selector, "selector")
    return _build_node("component", {"selector": validated_selector}, children)


def _build_representation(repr_type: str, color: Optional[str] = None, opacity: Optional[float] = None,
                         probe_radius: Optional[float] = None, **kwargs) -> Dict[str, Any]:
    """Build representation node with optional color, opacity, and surface-specific parameters."""
    validated_type = _validate_and_normalize(repr_type, "representation_type")
    params = {"type": validated_type}
    children = []
      
    # Add surface-specific parameters to params
    if probe_radius is not None and validated_type == "surface":
        validated_probe_radius = _validate_and_normalize(probe_radius, "probe_radius")
        params["probe_radius"] = validated_probe_radius
      
    # Add color and opacity as children nodes
    if color:
        validated_color = _validate_and_normalize(color, "color")
        children.append(_build_node("color", {"color": validated_color}))
    if opacity is not None:
        validated_opacity = _validate_and_normalize(opacity, "opacity")
        children.append(_build_node("opacity", {"opacity": validated_opacity}))
      
    return _build_node("representation", params, children if children else None)


def _build_focus(camera_angle: Optional[str] = None, radius_factor: Optional[float] = None, 
                interactions: bool = False) -> Dict[str, Any]:
    """Build focus node with camera and interaction options."""
    params = {}
      
    if radius_factor is not None:
        params["radius_factor"] = radius_factor
    if interactions:
        params["interactions"] = True
    if camera_angle:
        validated_angle = _validate_and_normalize(camera_angle, "camera_angle")
        direction, up = CAMERA_ANGLES[validated_angle]
        params.update({"direction": direction, "up": up})
      
    return _build_node("focus", params if params else None)


def _build_label(text: str) -> Dict[str, Any]:
    """Build label node with validated text."""
    validated_text = _validate_and_normalize(text, "text")
    return _build_node("label", {"text": validated_text})


def save_json(mvsj_data: Dict[str, Any], filename: str) -> None:
    """Save MVSJ data to JSON file."""
    dirname = os.path.dirname(filename)
    if dirname:
        os.makedirs(dirname, exist_ok=True)
    with open(filename, 'w') as f:
        json.dump(mvsj_data, f, indent=2)