"""
Documentation export utilities for MVS Story Generator.
"""
import os
import re
from pathlib import Path
from typing import Optional, List, Dict


def export_docs_to_txt(output_file: str = "mvs_story_generator_docs.txt", 
                      docs_dir: Optional[str] = None,
                      exclude_files: Optional[List[str]] = None) -> str:
    """
    Export all documentation files to a single text file for LLM consumption.
    
    Args:
        output_file: Output filename for the compiled documentation
        docs_dir: Path to the docs directory (auto-detected if None)
        exclude_files: List of files to exclude (default excludes installation.md)
        
    Returns:
        Path to the created text file
        
    Example:
        >>> from mvs_story_generator import export_docs_to_txt
        >>> doc_file = export_docs_to_txt("complete_docs.txt")
        >>> print(f"Documentation exported to: {doc_file}")
    """
    # Default exclusions
    if exclude_files is None:
        exclude_files = ["installation.md"]
    
    # Auto-detect docs directory if not provided
    if docs_dir is None:
        # Find docs directory relative to package
        package_dir = Path(__file__).parent.parent
        docs_dir = package_dir.parent / "docs"
        
        if not docs_dir.exists():
            raise FileNotFoundError(f"Documentation directory not found at {docs_dir}")
    else:
        docs_dir = Path(docs_dir)
    
    # Collect all markdown files
    md_files = _collect_markdown_files(docs_dir, exclude_files)
    
    # Create output file path
    if not os.path.isabs(output_file):
        output_file = os.path.join(os.getcwd(), output_file)
    
    # Process and write documentation
    with open(output_file, 'w', encoding='utf-8') as f:
        f.write("# MVS Story Generator - Complete Documentation\n")
        f.write("# Compiled for LLM Consumption\n")
        f.write("=" * 80 + "\n\n")
        
        for file_info in md_files:
            _write_file_section(f, file_info, docs_dir)
    
    return output_file


def _collect_markdown_files(docs_dir: Path, exclude_files: List[str]) -> List[Dict[str, str]]:
    """Collect and organize markdown files for processing."""
    files = []
    
    # Define file order for logical flow
    priority_order = [
        "index.md",
        "getting-started/quickstart.md", 
        "getting-started/examples.md",
        "api/index.md",
        "api/core.md",
        "api/visualization-basic.md",
        "api/visualization-surface.md", 
        "api/stories.md",
        "api/examples.md",
        "api/types.md"
    ]
    
    # Add priority files first
    for rel_path in priority_order:
        file_path = docs_dir / rel_path
        if file_path.exists() and rel_path.split('/')[-1] not in exclude_files:
            files.append({
                "path": str(file_path),
                "relative_path": rel_path,
                "title": _extract_title_from_path(rel_path)
            })
    
    # Add any remaining markdown files
    for md_file in docs_dir.rglob("*.md"):
        rel_path = md_file.relative_to(docs_dir)
        rel_path_str = str(rel_path).replace('\\', '/')
        
        # Skip if already added or excluded
        if (any(f["relative_path"] == rel_path_str for f in files) or 
            md_file.name in exclude_files):
            continue
            
        files.append({
            "path": str(md_file),
            "relative_path": rel_path_str,
            "title": _extract_title_from_path(rel_path_str)
        })
    
    return files


def _extract_title_from_path(rel_path: str) -> str:
    """Extract a readable title from file path."""
    # Remove extension and convert to title
    name = Path(rel_path).stem
    
    # Handle special cases
    title_map = {
        "index": "Overview",
        "quickstart": "Quick Start Guide",
        "examples": "Examples",
        "visualization-basic": "Basic Visualizations",
        "visualization-surface": "Surface Visualizations",
        "stories": "Multi-State Stories",
        "types": "Type Definitions",
        "core": "Core Functions"
    }
    
    if name in title_map:
        return title_map[name]
    
    # Convert kebab-case to title case
    return name.replace('-', ' ').replace('_', ' ').title()


def _write_file_section(f, file_info: Dict[str, str], docs_dir: Path) -> None:
    """Write a single file section to the output."""
    f.write(f"\n{'='*80}\n")
    f.write(f"# {file_info['title']}\n")
    f.write(f"# Source: {file_info['relative_path']}\n")
    f.write(f"{'='*80}\n\n")
    
    try:
        with open(file_info["path"], 'r', encoding='utf-8') as md_file:
            content = md_file.read()
            
        # Process markdown content for text format
        processed_content = _process_markdown_for_txt(content)
        f.write(processed_content)
        f.write("\n\n")
        
    except Exception as e:
        f.write(f"Error reading file: {e}\n\n")


def _process_markdown_for_txt(content: str) -> str:
    """Process markdown content for plain text consumption."""
    # Remove mkdocstrings directives (they won't render in plain text)
    content = re.sub(r'^:::\s+.*$', '[API Reference - see source code for details]', content, flags=re.MULTILINE)
    
    # Convert markdown headers to plain text headers
    content = re.sub(r'^(#{1,6})\s+(.+)$', lambda m: f"\n{m.group(2)}\n{'-' * len(m.group(2))}", content, flags=re.MULTILINE)
    
    # Convert code blocks to indented text
    content = re.sub(r'^```(\w+)?\n(.*?)^```', lambda m: _format_code_block(m.group(2)), content, flags=re.MULTILINE | re.DOTALL)
    
    # Convert inline code
    content = re.sub(r'`([^`]+)`', r'"\1"', content)
    
    # Convert markdown links to plain text
    content = re.sub(r'\[([^\]]+)\]\([^)]+\)', r'\1', content)
    
    # Convert bold/italic to plain text emphasis
    content = re.sub(r'\*\*([^*]+)\*\*', r'\1 (IMPORTANT)', content)
    content = re.sub(r'\*([^*]+)\*', r'\1', content)
    
    # Convert lists to plain text
    content = re.sub(r'^[-*+]\s+', '• ', content, flags=re.MULTILINE)
    content = re.sub(r'^\d+\.\s+', lambda m: f"{m.group()[:-1]} ", content, flags=re.MULTILINE)
    
    # Clean up excessive whitespace
    content = re.sub(r'\n{3,}', '\n\n', content)
    
    return content.strip()


def _format_code_block(code: str) -> str:
    """Format code block for plain text."""
    lines = code.strip().split('\n')
    formatted_lines = []
    for line in lines:
        formatted_lines.append(f"    {line}")
    return '\n' + '\n'.join(formatted_lines) + '\n'


def export_api_reference_only(output_file: str = "mvs_api_reference.txt") -> str:
    """
    Export only API reference documentation for focused LLM consumption.
    
    Args:
        output_file: Output filename for the API reference
        
    Returns:
        Path to the created text file
        
    Example:
        >>> from mvs_story_generator import export_api_reference_only
        >>> api_file = export_api_reference_only("api_only.txt")
    """
    package_dir = Path(__file__).parent.parent
    docs_dir = package_dir.parent / "docs"
    
    if not docs_dir.exists():
        raise FileNotFoundError(f"Documentation directory not found at {docs_dir}")
    
    # Only include API files
    api_files = [
        "api/index.md",
        "api/core.md", 
        "api/visualization-basic.md",
        "api/visualization-surface.md",
        "api/stories.md",
        "api/examples.md",
        "api/types.md"
    ]
    
    md_files = []
    for rel_path in api_files:
        file_path = docs_dir / rel_path
        if file_path.exists():
            md_files.append({
                "path": str(file_path),
                "relative_path": rel_path,
                "title": _extract_title_from_path(rel_path)
            })
    
    # Create output file
    if not os.path.isabs(output_file):
        output_file = os.path.join(os.getcwd(), output_file)
    
    with open(output_file, 'w', encoding='utf-8') as f:
        f.write("# MVS Story Generator - API Reference\n")
        f.write("# Compiled for LLM Consumption\n")
        f.write("=" * 80 + "\n\n")
        
        for file_info in md_files:
            _write_file_section(f, file_info, docs_dir)
    
    return output_file