"""
Validation utilities for MVS Story Generator.
"""
import re
from typing import Any, Dict, Union
from .constants import COLOR_NAMES, CAMERA_ANGLES


def _validate_and_normalize(value: Any, validator_type: str, **kwargs) -> Any:
    """Centralized validation for all input types."""
    if value is None:
        return None
       
    if validator_type == "color":
        value = str(value).lower().strip()
        if value in COLOR_NAMES:
            return COLOR_NAMES[value]
        if re.match(r'^#?[0-9a-f]{6}$', value):
            return value if value.startswith('#') else f"#{value}"
        raise ValueError(f"Invalid color: '{value}'")
       
    elif validator_type == "camera_angle":
        value = str(value).lower().strip()
        if value in CAMERA_ANGLES:
            return value
        raise ValueError(f"Invalid camera angle: '{value}'. Valid: {', '.join(CAMERA_ANGLES.keys())}")
       
    elif validator_type == "text":
        value = str(value).strip()
        if not value or len(value) > 200:
            raise ValueError("Text must be 1-200 characters")
        return ''.join(char for char in value if ord(char) >= 32 or char in '\n\t')
       
    elif validator_type == "selector":
        if isinstance(value, str):
            valid_static = {"all", "polymer", "protein", "nucleic", "branched", "ligand", "ion", "water"}
            if value.lower() in valid_static:
                return value.lower()
            raise ValueError(f"Invalid static selector: '{value}'")
        elif isinstance(value, dict):
            valid_keys = {"label_entity_id", "label_asym_id", "auth_asym_id", "label_seq_id",
                    "auth_seq_id", "pdbx_PDB_ins_code", "beg_label_seq_id", "end_label_seq_id",
                    "beg_auth_seq_id", "end_auth_seq_id", "label_atom_id", "auth_atom_id",
                    "type_symbol", "atom_id", "atom_index", "instance_id"}
            unknown_keys = set(value.keys()) - valid_keys
            if unknown_keys:
                raise ValueError(f"Invalid ComponentExpression keys: {unknown_keys}")
            return value
        raise ValueError(f"Selector must be string or dictionary, got {type(value)}")
      
    elif validator_type == "opacity":
        try:
            value = float(value)
            if not 0.0 <= value <= 1.0:
                raise ValueError("Opacity must be between 0.0 and 1.0")
            return value
        except (ValueError, TypeError):
            raise ValueError(f"Invalid opacity: '{value}'. Must be a number between 0.0 and 1.0")
      
    elif validator_type == "probe_radius":
        try:
            value = float(value)
            if not 0.1 <= value <= 5.0:
                raise ValueError("Probe radius must be between 0.1 and 5.0")
            return value
        except (ValueError, TypeError):
            raise ValueError(f"Invalid probe radius: '{value}'. Must be a number between 0.1 and 5.0")
      
    elif validator_type == "representation_type":
        valid_types = {"cartoon", "ball_and_stick", "surface", "spacefill", "ribbon", "tube", "line"}
        value = str(value).lower().strip()
        if value in valid_types:
            return value
        raise ValueError(f"Invalid representation type: '{value}'. Valid: {', '.join(sorted(valid_types))}")
      
    return value