"""
Example generation for MVS Story Generator.
"""
from typing import List, Tuple
from ..core.builders import save_json
from ..visualization import (
    protein_cartoon, ligand_view, multi_component_colored_view,
    protein_surface, ligand_surface, protein_surface_with_cartoon, ligand_surface_with_sticks
)
from ..stories import (
    protein_rotation_story, structure_comparison_story,
    protein_surface_evolution_story, ligand_representation_transition_story,
    protein_ligand_surface_story, hybrid_representation_showcase_story
)


def generate_examples(example_type: str, output_dir: str = None, **kwargs) -> None:
    """Generate examples using data-driven approach."""
      
    if example_type == "kinases":
        # Kinase data
        kinases = [
            ("1ATP", "PKA", "cAMP-dependent protein kinase"),
            ("1IRK", "InsR", "Insulin receptor kinase"),
            ("1FMK", "CDK2", "Cyclin-dependent kinase 2"),
            ("2SRC", "c-Src", "c-Src non-receptor tyrosine kinase"),
            ("1CSK", "CSK", "C-terminal Src kinase"),
            ("3PY3", "p38", "p38 MAP kinase"),
            ("2ERK", "ERK2", "ERK2/MAPK1"),
            ("1JNK", "JNK1", "JNK1 stress-activated protein kinase"),
            ("4E4N", "Aurora-B", "Aurora B mitotic kinase"),
            ("2WEL", "GSK3β", "Glycogen synthase kinase 3 beta")
        ]
           
        output_dir = output_dir or "kinase_protein"
        include_variants = kwargs.get("include_variants", False)
           
        print(f"Generating {len(kinases)} kinase MVSJ files...")
           
        for pdb_id, short_name, description in kinases:
            url = f"https://www.ebi.ac.uk/pdbe/entry-files/{pdb_id.lower()}.bcif"
            title = f"{pdb_id} {short_name} - {description}"
            filename = f"{output_dir}/{pdb_id}_{short_name}_cartoon.mvsj"
                
            mvsj_data = protein_cartoon(url, title, kwargs.get("color"))
            save_json(mvsj_data, filename)
            print(f" → Saved: {filename}")
                
            # Generate variants if requested
            if include_variants:
                variants = [("blue", "blue"), ("red", "red"), ("green", "green")]
                for color_name, color_value in variants:
                    variant_filename = f"{output_dir}/{pdb_id}_{short_name}_cartoon_{color_name}.mvsj"
                    variant_mvsj = protein_cartoon(url, f"{title} ({color_name.title()})", color_value)
                    save_json(variant_mvsj, variant_filename)
                    print(f" → {color_name} variant: {variant_filename}")
      
    elif example_type == "multi_state_stories":
        output_dir = output_dir or "multi_state_stories"
           
        # Story configurations
        stories = [
            ("PKA Rotation", "protein_rotation", {
                "url": "https://www.ebi.ac.uk/pdbe/entry-files/1atp.bcif",
                "camera_angles": ["up", "down", "left", "right", "flip_horizontal", "flip_vertical"]
            }),
            ("Kinase Comparison", "structure_comparison", {
                "urls_and_colors": [
                    ("https://www.ebi.ac.uk/pdbe/entry-files/1atp.bcif", "blue"),
                    ("https://www.ebi.ac.uk/pdbe/entry-files/2src.bcif", "red"),
                    ("https://www.ebi.ac.uk/pdbe/entry-files/1csk.bcif", "green")
                ]
            })
        ]
           
        print(f"Generating {len(stories)} multi-state stories...")
           
        for i, (story_name, story_type, config) in enumerate(stories, 1):
            filename = f"{output_dir}/{i:02d}_{story_name.replace(' ', '_').lower()}.mvsj"
                
            if story_type == "protein_rotation":
                story_data = protein_rotation_story(**config, title=story_name)
            elif story_type == "structure_comparison":
                story_data = structure_comparison_story(**config, title=story_name)
                
            save_json(story_data, filename)
            print(f" → Saved: {filename}")
      
    elif example_type == "surface_examples":
        output_dir = output_dir or "surface_examples"
           
        # Surface example configurations
        surface_examples = [
            ("Protein Surface", "protein_surface", {
                "url": "https://www.ebi.ac.uk/pdbe/entry-files/1atp.bcif",
                "color": "#87CEEB", "opacity": 0.7
            }),
            ("Ligand Surface", "ligand_surface", {
                "url": "https://www.ebi.ac.uk/pdbe/entry-files/1atp.bcif",
                "color": "#90EE90", "opacity": 0.6
            }),
            ("Protein Surface with Cartoon", "protein_surface_with_cartoon", {
                "url": "https://www.ebi.ac.uk/pdbe/entry-files/1atp.bcif",
                "cartoon_color": "#4577B2", "surface_color": "#D0D0D0", "surface_opacity": 0.4
            }),
            ("Ligand Surface with Sticks", "ligand_surface_with_sticks", {
                "url": "https://www.ebi.ac.uk/pdbe/entry-files/1atp.bcif",
                "sticks_color": "#FF6B6B", "surface_color": "#90EE90", "surface_opacity": 0.5
            })
        ]
           
        print(f"Generating {len(surface_examples)} surface visualization examples...")
           
        for i, (example_name, example_type_func, config) in enumerate(surface_examples, 1):
            filename = f"{output_dir}/{i:02d}_{example_name.replace(' ', '_').lower()}.mvsj"
                
            # Get the function by name
            if example_type_func == "protein_surface":
                example_data = protein_surface(**config, title=example_name)
            elif example_type_func == "ligand_surface":
                example_data = ligand_surface(**config, title=example_name)
            elif example_type_func == "protein_surface_with_cartoon":
                example_data = protein_surface_with_cartoon(**config, title=example_name)
            elif example_type_func == "ligand_surface_with_sticks":
                example_data = ligand_surface_with_sticks(**config, title=example_name)
                
            save_json(example_data, filename)
            print(f" → Saved: {filename}")
      
    elif example_type == "surface_stories":
        output_dir = output_dir or "surface_stories"
           
        # Surface story configurations
        surface_stories = [
            ("Protein Surface Evolution", "protein_surface_evolution_story", {
                "url": "https://www.ebi.ac.uk/pdbe/entry-files/1atp.bcif"
            }),
            ("Ligand Representation Transition", "ligand_representation_transition_story", {
                "url": "https://www.ebi.ac.uk/pdbe/entry-files/1atp.bcif"
            }),
            ("Protein-Ligand Surface Interaction", "protein_ligand_surface_story", {
                "url": "https://www.ebi.ac.uk/pdbe/entry-files/1atp.bcif"
            }),
            ("Hybrid Representation Showcase", "hybrid_representation_showcase_story", {
                "url": "https://www.ebi.ac.uk/pdbe/entry-files/1atp.bcif"
            })
        ]
           
        print(f"Generating {len(surface_stories)} surface multi-state stories...")
           
        for i, (story_name, story_type, config) in enumerate(surface_stories, 1):
            filename = f"{output_dir}/{i:02d}_{story_name.replace(' ', '_').replace('-', '_').lower()}.mvsj"
                
            if story_type == "protein_surface_evolution_story":
                story_data = protein_surface_evolution_story(**config, title=story_name)
            elif story_type == "ligand_representation_transition_story":
                story_data = ligand_representation_transition_story(**config, title=story_name)
            elif story_type == "protein_ligand_surface_story":
                story_data = protein_ligand_surface_story(**config, title=story_name)
            elif story_type == "hybrid_representation_showcase_story":
                story_data = hybrid_representation_showcase_story(**config, title=story_name)
                
            save_json(story_data, filename)
            print(f" → Saved: {filename}")
      
    print(f"✅ Example generation completed!")