"""
Convenience functions for creating common multi-state stories.
"""
from typing import Dict, Any, Optional, List, Tuple
from ..core.constants import DEFAULT_COLORS
from .multistate import create_multi_state_story


def protein_rotation_story(url: str, color: Optional[str] = None,
                          camera_angles: Optional[List[str]] = None,
                          title: Optional[str] = None) -> Dict[str, Any]:
    """Create protein rotation story."""
    angles = camera_angles or ["up", "right", "flip_horizontal"]
    configs = []
      
    angle_titles = {
        "up": "Rotated Up", "down": "Rotated Down", "left": "Rotated Left",
        "right": "Rotated Right", "flip_horizontal": "180° Horizontal",
        "flip_vertical": "180° Vertical"
    }
      
    for angle in angles:
        configs.append({
            "url": url, "viz_type": "protein_cartoon", "color": color or DEFAULT_COLORS["protein"],
            "camera_angle": angle, "title": angle_titles.get(angle, f"Angle {angle}")
        })
      
    return create_multi_state_story(configs, title or f"Protein Rotation - {len(angles)} Views")


def structure_comparison_story(urls_and_colors: List[Tuple[str, str]], viz_type: str = "protein_cartoon",
                             title: Optional[str] = None) -> Dict[str, Any]:
    """Create structure comparison story."""
    configs = []
    for i, (url, color) in enumerate(urls_and_colors):
        configs.append({
            "url": url, "viz_type": viz_type, "color": color,
            "title": f"Structure {i+1} ({color})"
        })
      
    return create_multi_state_story(configs, title or f"Structure Comparison - {len(urls_and_colors)} Structures")


def protein_surface_evolution_story(url: str, title: Optional[str] = None,
                                   surface_opacities: Optional[List[float]] = None,
                                   colors: Optional[List[str]] = None) -> Dict[str, Any]:
    """Create protein surface evolution story showing different opacity levels."""
    opacities = surface_opacities or [0.2, 0.4, 0.6, 0.8, 1.0]
    story_colors = colors or ["#D0D0D0", "#B0C4DE", "#87CEEB", "#4169E1", "#0000CD"]
      
    configs = []
    for i, (opacity, color) in enumerate(zip(opacities, story_colors)):
        configs.append({
            "url": url,
            "viz_type": "protein_surface",
            "color": color,
            "surface_opacity": opacity,
            "title": f"Surface Opacity {opacity:.1f}",
            "duration": 2500
        })
      
    return create_multi_state_story(configs, title or "Protein Surface Evolution")


def ligand_representation_transition_story(url: str, title: Optional[str] = None,
                                         ligand_color: Optional[str] = None,
                                         surface_color: Optional[str] = None) -> Dict[str, Any]:
    """Create ligand representation transition story: sticks → surface → hybrid."""
    base_color = ligand_color or DEFAULT_COLORS["ligand"]
    surf_color = surface_color or "#90EE90"
      
    configs = [
        {
            "url": url,
            "viz_type": "ligand_view",
            "color": base_color,
            "title": "Ball and Stick",
            "duration": 3000
        },
        {
            "url": url,
            "viz_type": "ligand_surface",
            "color": surf_color,
            "surface_opacity": 0.7,
            "title": "Surface Only",
            "duration": 3000
        },
        {
            "url": url,
            "viz_type": "ligand_surface_with_sticks",
            "color": base_color,
            "surface_color": surf_color,
            "surface_opacity": 0.5,
            "title": "Surface + Sticks",
            "duration": 3000
        }
    ]
      
    return create_multi_state_story(configs, title or "Ligand Representation Transition")


def protein_ligand_surface_story(url: str, title: Optional[str] = None,
                                protein_color: Optional[str] = None,
                                ligand_color: Optional[str] = None) -> Dict[str, Any]:
    """Create protein-ligand surface interaction story."""
    prot_color = protein_color or DEFAULT_COLORS["protein"]
    lig_color = ligand_color or DEFAULT_COLORS["ligand"]
      
    configs = [
        {
            "url": url,
            "viz_type": "multi_component",
            "color": prot_color,
            "title": "Standard View",
            "duration": 2500
        },
        {
            "url": url,
            "viz_type": "multi_component",
            "color": prot_color,
            "include_protein_surface": True,
            "protein_surface_opacity": 0.3,
            "title": "Protein Surface Added",
            "duration": 2500
        },
        {
            "url": url,
            "viz_type": "multi_component",
            "color": prot_color,
            "include_protein_surface": True,
            "protein_surface_opacity": 0.3,
            "include_ligand_surface": True,
            "ligand_surface_opacity": 0.6,
            "title": "Both Surfaces",
            "duration": 3000
        },
        {
            "url": url,
            "viz_type": "multi_component_ligand_focus",
            "ligand_color": lig_color,
            "include_protein_surface": True,
            "protein_surface_opacity": 0.2,
            "include_ligand_surface": True,
            "ligand_surface_opacity": 0.7,
            "title": "Surface Interaction Focus",
            "duration": 3000
        }
    ]
      
    return create_multi_state_story(configs, title or "Protein-Ligand Surface Interaction")


def hybrid_representation_showcase_story(url: str, title: Optional[str] = None) -> Dict[str, Any]:
    """Create showcase story demonstrating all hybrid surface representations."""
    configs = [
        {
            "url": url,
            "viz_type": "protein_cartoon",
            "color": "#4577B2",
            "title": "Standard Cartoon",
            "duration": 2000
        },
        {
            "url": url,
            "viz_type": "protein_surface",
            "color": "#87CEEB",
            "surface_opacity": 0.8,
            "title": "Surface Only",
            "duration": 2000
        },
        {
            "url": url,
            "viz_type": "protein_surface_with_cartoon",
            "cartoon_color": "#4577B2",
            "surface_color": "#87CEEB",
            "surface_opacity": 0.4,
            "title": "Hybrid: Cartoon + Surface",
            "duration": 2500
        },
        {
            "url": url,
            "viz_type": "ligand_view",
            "color": "#FF6B6B",
            "title": "Ligand Ball & Stick",
            "duration": 2000
        },
        {
            "url": url,
            "viz_type": "ligand_surface",
            "color": "#90EE90",
            "surface_opacity": 0.7,
            "title": "Ligand Surface",
            "duration": 2000
        },
        {
            "url": url,
            "viz_type": "ligand_surface_with_sticks",
            "sticks_color": "#FF6B6B",
            "surface_color": "#90EE90",
            "surface_opacity": 0.5,
            "title": "Hybrid: Sticks + Surface",
            "duration": 2500
        }
    ]
      
    return create_multi_state_story(configs, title or "Hybrid Representation Showcase")