"""
Multi-state story creation for MVS Story Generator.
"""
from typing import Dict, Any, Optional, List
from ..core.types import StateConfig
from ..core.constants import DEFAULT_STATE_DURATION, DEFAULT_TRANSITION_DURATION
from ..core.builders import _build_multistate_base, _build_node


def _wrap_in_snapshot(state_tree: Dict, title: Optional[str] = None, duration: Optional[int] = None,
                     transition_duration: Optional[int] = None, key: Optional[str] = None) -> Dict[str, Any]:
    """Wrap state tree in snapshot with timing metadata."""
    metadata = {
        "linger_duration_ms": duration or DEFAULT_STATE_DURATION,
        "transition_duration_ms": transition_duration or DEFAULT_TRANSITION_DURATION
    }
    if title:
        metadata["title"] = title
    if key:
        metadata["key"] = key
      
    enhanced_tree = {
        "kind": "root",
        "children": state_tree["children"] + [_build_node("canvas", {"background_color": "white"})]
    }
      
    return {"metadata": metadata, "root": enhanced_tree}


def create_multi_state_story(state_configs: List[StateConfig], title: Optional[str] = None) -> Dict[str, Any]:
    """Create multi-state molecular story from configurations with surface support."""
    if not state_configs:
        raise ValueError("state_configs cannot be empty")
      
    multi_state_mvsj = _build_multistate_base(title or "Multi-State Molecular Story")
      
    # Import visualization functions
    from ..visualization.basic import protein_cartoon, ligand_view, multi_component_colored_view, multi_component_ligand_focus
    from ..visualization.surface import protein_surface, ligand_surface, protein_surface_with_cartoon, ligand_surface_with_sticks
    
    # Map visualization types to functions
    viz_functions = {
        "protein_cartoon": protein_cartoon,
        "ligand_view": ligand_view,
        "multi_component": multi_component_colored_view,
        "multi_component_ligand_focus": multi_component_ligand_focus,
        "protein_surface": protein_surface,
        "ligand_surface": ligand_surface,
        "protein_surface_with_cartoon": protein_surface_with_cartoon,
        "ligand_surface_with_sticks": ligand_surface_with_sticks
    }
      
    for i, config in enumerate(state_configs):
        if config["viz_type"] not in viz_functions:
            raise ValueError(f"Invalid viz_type '{config['viz_type']}' in state {i}. Valid types: {', '.join(viz_functions.keys())}")
           
        viz_function = viz_functions[config["viz_type"]]
           
        # Prepare function arguments based on visualization type
        args = {"url": config["url"], "title": config.get("title"), "labels": config.get("labels")}
           
        # Handle color arguments based on visualization type
        if config["viz_type"] == "multi_component":
            args["protein_color"] = config.get("color")
            # Add surface parameters for multi_component
            args["include_protein_surface"] = config.get("include_protein_surface")
            args["protein_surface_color"] = config.get("protein_surface_color")
            args["protein_surface_opacity"] = config.get("protein_surface_opacity")
            args["include_ligand_surface"] = config.get("include_ligand_surface")
            args["ligand_surface_color"] = config.get("ligand_surface_color")
            args["ligand_surface_opacity"] = config.get("ligand_surface_opacity")
        elif config["viz_type"] == "multi_component_ligand_focus":
            args["ligand_color"] = config.get("color")
            # Add surface parameters for multi_component_ligand_focus
            args["include_protein_surface"] = config.get("include_protein_surface")
            args["protein_surface_color"] = config.get("protein_surface_color")
            args["protein_surface_opacity"] = config.get("protein_surface_opacity")
            args["include_ligand_surface"] = config.get("include_ligand_surface")
            args["ligand_surface_color"] = config.get("ligand_surface_color")
            args["ligand_surface_opacity"] = config.get("ligand_surface_opacity")
        elif config["viz_type"] == "protein_surface_with_cartoon":
            args["cartoon_color"] = config.get("color")
            args["surface_color"] = config.get("surface_color")
            args["surface_opacity"] = config.get("surface_opacity")
        elif config["viz_type"] == "ligand_surface_with_sticks":
            args["sticks_color"] = config.get("color")
            args["surface_color"] = config.get("surface_color")
            args["surface_opacity"] = config.get("surface_opacity")
        else:
            args["color"] = config.get("color")
            if "surface" in config["viz_type"]:
                args["opacity"] = config.get("surface_opacity")
           
        # Add common parameters
        args["camera_angle"] = config.get("camera_angle")
        args["probe_radius"] = config.get("probe_radius")
           
        # Generate single-state view and wrap in snapshot
        single_state = viz_function(**{k: v for k, v in args.items() if v is not None})
        snapshot = _wrap_in_snapshot(
            single_state["root"],
            config.get("title"),
            config.get("duration"),
            config.get("transition_duration"),
            f"state_{i+1}"
        )
        multi_state_mvsj["snapshots"].append(snapshot)
      
    return multi_state_mvsj