"""
Tests for basic visualization functions based on examples from the original code.
"""
import pytest
from mvs_story_generator.visualization import (
    protein_cartoon, ligand_view, multi_component_colored_view, multi_component_ligand_focus
)


class TestBasicVisualization:
    """Test basic visualization functions."""
    
    def setup_method(self):
        """Set up test fixtures."""
        self.test_url = "https://www.ebi.ac.uk/pdbe/entry-files/1atp.bcif"
    
    def test_protein_cartoon_basic(self):
        """Test basic protein cartoon visualization."""
        result = protein_cartoon(self.test_url, "Test Protein", color="blue")
        
        assert "metadata" in result
        assert "root" in result
        assert result["metadata"]["version"] == "1"
        assert result["metadata"]["title"] == "Test Protein"
        assert result["root"]["kind"] == "root"
        assert len(result["root"]["children"]) == 1
        
    def test_protein_cartoon_with_surface(self):
        """Test protein cartoon with surface representation."""
        result = protein_cartoon(
            self.test_url, 
            "Test Protein with Surface",
            color="blue",
            include_surface=True,
            surface_opacity=0.3
        )
        
        assert "metadata" in result
        assert result["metadata"]["title"] == "Test Protein with Surface"
        
    def test_ligand_view_basic(self):
        """Test basic ligand view visualization."""
        result = ligand_view(self.test_url, "Test Ligand", color="green")
        
        assert "metadata" in result
        assert "root" in result
        assert result["metadata"]["title"] == "Test Ligand"
        
    def test_ligand_view_with_surface(self):
        """Test ligand view with surface representation.""" 
        result = ligand_view(
            self.test_url,
            "Test Ligand with Surface",
            color="green",
            include_surface=True,
            surface_opacity=0.5
        )
        
        assert "metadata" in result
        assert result["metadata"]["title"] == "Test Ligand with Surface"
        
    def test_multi_component_colored_view(self):
        """Test multi-component colored visualization."""
        result = multi_component_colored_view(
            self.test_url,
            "Test Multi-Component",
            protein_color="#e19039",
            nucleic_color="#4b7fcc",
            ligand_color="#229954"
        )
        
        assert "metadata" in result
        assert result["metadata"]["title"] == "Test Multi-Component"
        
    def test_multi_component_with_surfaces(self):
        """Test multi-component with surface representations."""
        result = multi_component_colored_view(
            self.test_url,
            "Test Multi with Surfaces",
            include_protein_surface=True,
            protein_surface_opacity=0.3,
            include_ligand_surface=True,
            ligand_surface_opacity=0.6
        )
        
        assert "metadata" in result
        assert result["metadata"]["title"] == "Test Multi with Surfaces"
        
    def test_multi_component_ligand_focus(self):
        """Test multi-component ligand focus visualization."""
        result = multi_component_ligand_focus(
            self.test_url,
            "Test Ligand Focus",
            protein_color="#B0B0B0",
            ligand_color="#FF6B6B"
        )
        
        assert "metadata" in result
        assert result["metadata"]["title"] == "Test Ligand Focus"
        
    def test_highlighted_residues(self):
        """Test multi-component view with highlighted residues."""
        highlight_residues = [("A", 72), ("B", 95)]
        result = multi_component_colored_view(
            self.test_url,
            "Test with Highlights",
            highlight_residues=highlight_residues,
            highlight_color="#ff0000"
        )
        
        assert "metadata" in result
        assert result["metadata"]["title"] == "Test with Highlights"
        
    def test_camera_angles(self):
        """Test different camera angles."""
        camera_angles = ["up", "down", "left", "right", "flip_horizontal", "flip_vertical"]
        
        for angle in camera_angles:
            result = protein_cartoon(
                self.test_url,
                f"Test {angle}",
                camera_angle=angle
            )
            assert "metadata" in result
            assert result["metadata"]["title"] == f"Test {angle}"
            
    def test_labels(self):
        """Test visualization with labels."""
        labels = [
            {"text": "Active Site", "selector": "ligand"},
            {"text": "Important Residue", "selector": {"label_asym_id": "A", "label_seq_id": 72}}
        ]
        
        result = protein_cartoon(
            self.test_url,
            "Test with Labels",
            labels=labels
        )
        
        assert "metadata" in result
        assert result["metadata"]["title"] == "Test with Labels"