"""
Tests for documentation export functionality.
"""
import os
import tempfile
import pytest
from mvs_story_generator.core.docs_export import export_docs_to_txt, export_api_reference_only


class TestDocsExport:
    """Test documentation export functions."""
    
    def test_export_docs_to_txt(self):
        """Test full documentation export."""
        with tempfile.TemporaryDirectory() as temp_dir:
            output_file = os.path.join(temp_dir, "test_docs.txt")
            
            # Export documentation
            result_path = export_docs_to_txt(output_file)
            
            assert result_path == output_file
            assert os.path.exists(output_file)
            
            # Check file content
            with open(output_file, 'r', encoding='utf-8') as f:
                content = f.read()
                
            assert "MVS Story Generator - Complete Documentation" in content
            assert "Compiled for LLM Consumption" in content
            assert "Overview" in content
            assert "Quick Start Guide" in content
            assert "API Reference" in content
            
            # Should not contain installation.md content
            assert "pip install mvs-story-generator" not in content
            
    def test_export_api_reference_only(self):
        """Test API-only documentation export."""
        with tempfile.TemporaryDirectory() as temp_dir:
            output_file = os.path.join(temp_dir, "test_api.txt")
            
            # Export API documentation
            result_path = export_api_reference_only(output_file)
            
            assert result_path == output_file
            assert os.path.exists(output_file)
            
            # Check file content
            with open(output_file, 'r', encoding='utf-8') as f:
                content = f.read()
                
            assert "MVS Story Generator - API Reference" in content
            assert "Basic Visualizations" in content
            assert "Surface Visualizations" in content
            assert "Multi-State Stories" in content
            
    def test_file_size_difference(self):
        """Test that API-only export is smaller than full export."""
        with tempfile.TemporaryDirectory() as temp_dir:
            full_file = os.path.join(temp_dir, "full_docs.txt")
            api_file = os.path.join(temp_dir, "api_docs.txt")
            
            export_docs_to_txt(full_file)
            export_api_reference_only(api_file)
            
            full_size = os.path.getsize(full_file)
            api_size = os.path.getsize(api_file)
            
            # API-only should be smaller
            assert api_size < full_size
            
    def test_markdown_processing(self):
        """Test that markdown is properly converted to plain text."""
        with tempfile.TemporaryDirectory() as temp_dir:
            output_file = os.path.join(temp_dir, "test_processing.txt")
            
            export_docs_to_txt(output_file)
            
            with open(output_file, 'r', encoding='utf-8') as f:
                content = f.read()
                
            # Should not contain raw markdown syntax
            assert "```python" not in content
            assert "**" not in content
            assert "##" not in content
            
            # Should contain converted content
            assert "[API Reference - see source code for details]" in content
            
    def test_exclude_files_parameter(self):
        """Test that exclude_files parameter works."""
        with tempfile.TemporaryDirectory() as temp_dir:
            output_file = os.path.join(temp_dir, "test_exclude.txt")
            
            # Export with additional exclusions
            export_docs_to_txt(output_file, exclude_files=["installation.md", "quickstart.md"])
            
            with open(output_file, 'r', encoding='utf-8') as f:
                content = f.read()
                
            # Should not contain quickstart content
            assert "Quick Start Guide" not in content
            
    def test_missing_docs_directory(self):
        """Test error handling for missing docs directory."""
        with pytest.raises(FileNotFoundError, match="Documentation directory not found"):
            export_docs_to_txt("test.txt", docs_dir="/nonexistent/path")