"""
Tests for example generation based on examples from the original code.
"""
import pytest
import tempfile
import os
import json
from mvs_story_generator.examples import generate_examples


class TestExampleGeneration:
    """Test example generation functions."""
    
    def test_kinases_example_generation(self):
        """Test kinase example generation based on original code."""
        with tempfile.TemporaryDirectory() as temp_dir:
            # Capture print output by redirecting to variable
            import io
            import sys
            captured_output = io.StringIO()
            sys.stdout = captured_output
            
            try:
                generate_examples("kinases", output_dir=temp_dir)
            finally:
                sys.stdout = sys.__stdout__
            
            output = captured_output.getvalue()
            
            # Check that kinase files were generated
            assert "Generating 10 kinase MVSJ files" in output
            assert "✅ Example generation completed!" in output
            
            # Check that some files were created
            files = os.listdir(temp_dir)
            assert len(files) > 0
            
            # Check that at least one file is valid JSON
            for filename in files[:3]:  # Check first 3 files
                if filename.endswith('.mvsj'):
                    filepath = os.path.join(temp_dir, filename)
                    with open(filepath, 'r') as f:
                        data = json.load(f)
                        assert "metadata" in data
                        assert "root" in data
                        
    def test_kinases_with_variants(self):
        """Test kinase generation with color variants."""
        with tempfile.TemporaryDirectory() as temp_dir:
            import io
            import sys
            captured_output = io.StringIO()
            sys.stdout = captured_output
            
            try:
                generate_examples("kinases", output_dir=temp_dir, include_variants=True)
            finally:
                sys.stdout = sys.__stdout__
            
            output = captured_output.getvalue()
            
            # Should have variant files
            assert "blue variant:" in output
            assert "red variant:" in output
            assert "green variant:" in output
            
    def test_multi_state_stories_generation(self):
        """Test multi-state story generation."""
        with tempfile.TemporaryDirectory() as temp_dir:
            import io
            import sys
            captured_output = io.StringIO()
            sys.stdout = captured_output
            
            try:
                generate_examples("multi_state_stories", output_dir=temp_dir)
            finally:
                sys.stdout = sys.__stdout__
            
            output = captured_output.getvalue()
            
            assert "Generating 2 multi-state stories" in output
            assert "PKA Rotation" in output or "pka_rotation" in output
            assert "Kinase Comparison" in output or "kinase_comparison" in output
            
    def test_surface_examples_generation(self):
        """Test surface example generation."""
        with tempfile.TemporaryDirectory() as temp_dir:
            import io
            import sys
            captured_output = io.StringIO()
            sys.stdout = captured_output
            
            try:
                generate_examples("surface_examples", output_dir=temp_dir)
            finally:
                sys.stdout = sys.__stdout__
            
            output = captured_output.getvalue()
            
            assert "Generating 4 surface visualization examples" in output
            assert "Protein Surface" in output
            assert "Ligand Surface" in output
            
    def test_surface_stories_generation(self):
        """Test surface story generation."""
        with tempfile.TemporaryDirectory() as temp_dir:
            import io
            import sys
            captured_output = io.StringIO()
            sys.stdout = captured_output
            
            try:
                generate_examples("surface_stories", output_dir=temp_dir)
            finally:
                sys.stdout = sys.__stdout__
            
            output = captured_output.getvalue()
            
            assert "Generating 4 surface multi-state stories" in output
            assert "Surface Evolution" in output or "surface_evolution" in output
            assert "Representation Transition" in output or "representation_transition" in output
            
    def test_generated_file_structure(self):
        """Test that generated files have correct structure."""
        with tempfile.TemporaryDirectory() as temp_dir:
            # Redirect stdout to suppress print statements
            import io
            import sys
            captured_output = io.StringIO()
            sys.stdout = captured_output
            
            try:
                generate_examples("kinases", output_dir=temp_dir)
            finally:
                sys.stdout = sys.__stdout__
            
            # Find a generated file
            files = [f for f in os.listdir(temp_dir) if f.endswith('.mvsj')]
            assert len(files) > 0
            
            # Load and validate structure
            with open(os.path.join(temp_dir, files[0]), 'r') as f:
                data = json.load(f)
                
            # Basic MVSJ structure validation
            assert "metadata" in data
            assert "root" in data
            assert data["metadata"]["version"] == "1"
            assert "timestamp" in data["metadata"]
            assert data["root"]["kind"] == "root"
            assert "children" in data["root"]
            
    def test_kinase_data_integrity(self):
        """Test that kinase data matches original examples."""
        with tempfile.TemporaryDirectory() as temp_dir:
            # Redirect stdout
            import io
            import sys
            captured_output = io.StringIO()
            sys.stdout = captured_output
            
            try:
                generate_examples("kinases", output_dir=temp_dir)
            finally:
                sys.stdout = sys.__stdout__
            
            output = captured_output.getvalue()
            
            # Check for specific kinases from original data
            expected_kinases = ["1ATP", "PKA", "1IRK", "InsR", "2SRC", "c-Src"]
            for kinase in expected_kinases:
                assert kinase in output
                
    def test_url_format_in_generated_files(self):
        """Test that URLs are correctly formatted in generated files."""
        with tempfile.TemporaryDirectory() as temp_dir:
            # Redirect stdout
            import io
            import sys
            captured_output = io.StringIO()
            sys.stdout = captured_output
            
            try:
                generate_examples("kinases", output_dir=temp_dir)
            finally:
                sys.stdout = sys.__stdout__
            
            files = [f for f in os.listdir(temp_dir) if f.endswith('.mvsj')]
            
            # Check first file's URL structure
            with open(os.path.join(temp_dir, files[0]), 'r') as f:
                data = json.load(f)
                
            # Navigate to download node
            download_node = data["root"]["children"][0]
            assert download_node["kind"] == "download"
            assert "url" in download_node["params"]
            assert download_node["params"]["url"].startswith("https://www.ebi.ac.uk/pdbe/entry-files/")
            assert download_node["params"]["url"].endswith(".bcif")