"""
Tests for multi-state story functions based on examples from the original code.
"""
import pytest
from mvs_story_generator.stories import (
    create_multi_state_story,
    protein_rotation_story,
    structure_comparison_story,
    protein_surface_evolution_story,
    ligand_representation_transition_story,
    protein_ligand_surface_story,
    hybrid_representation_showcase_story
)


class TestMultiStateStories:
    """Test multi-state story functions."""
    
    def setup_method(self):
        """Set up test fixtures."""
        self.test_url = "https://www.ebi.ac.uk/pdbe/entry-files/1atp.bcif"
    
    def test_create_multi_state_story_basic(self):
        """Test basic multi-state story creation."""
        configs = [
            {"url": self.test_url, "viz_type": "protein_cartoon", "color": "blue", "title": "Overview"},
            {"url": self.test_url, "viz_type": "ligand_view", "color": "red", "title": "Ligand Focus"}
        ]
        
        result = create_multi_state_story(configs, "Test Story")
        
        assert "metadata" in result
        assert "snapshots" in result
        assert result["kind"] == "multiple"
        assert result["metadata"]["title"] == "Test Story"
        assert len(result["snapshots"]) == 2
        
    def test_create_multi_state_story_with_surfaces(self):
        """Test multi-state story with surface visualizations."""
        configs = [
            {"url": self.test_url, "viz_type": "protein_cartoon", "color": "blue", "title": "Cartoon"},
            {"url": self.test_url, "viz_type": "protein_surface", "color": "#87CEEB", "surface_opacity": 0.7, "title": "Surface"},
            {"url": self.test_url, "viz_type": "protein_surface_with_cartoon", "color": "blue", 
             "surface_color": "#D0D0D0", "surface_opacity": 0.4, "title": "Hybrid"}
        ]
        
        result = create_multi_state_story(configs, "Surface Story")
        
        assert len(result["snapshots"]) == 3
        assert result["metadata"]["title"] == "Surface Story"
        
    def test_protein_rotation_story(self):
        """Test protein rotation story based on original examples."""
        # Based on the PKA rotation example from the original code
        result = protein_rotation_story(
            self.test_url,
            camera_angles=["up", "down", "left", "right", "flip_horizontal", "flip_vertical"]
        )
        
        assert "snapshots" in result
        assert len(result["snapshots"]) == 6
        assert result["metadata"]["title"] == "Protein Rotation - 6 Views"
        
    def test_structure_comparison_story(self):
        """Test structure comparison story based on original examples."""
        # Based on the kinase comparison example from the original code
        urls_and_colors = [
            ("https://www.ebi.ac.uk/pdbe/entry-files/1atp.bcif", "blue"),
            ("https://www.ebi.ac.uk/pdbe/entry-files/2src.bcif", "red"),
            ("https://www.ebi.ac.uk/pdbe/entry-files/1csk.bcif", "green")
        ]
        
        result = structure_comparison_story(urls_and_colors, title="Kinase Comparison")
        
        assert len(result["snapshots"]) == 3
        assert result["metadata"]["title"] == "Kinase Comparison"
        
    def test_protein_surface_evolution_story(self):
        """Test protein surface evolution story."""
        result = protein_surface_evolution_story(self.test_url)
        
        assert "snapshots" in result
        assert len(result["snapshots"]) == 5  # Default opacity levels
        assert result["metadata"]["title"] == "Protein Surface Evolution"
        
    def test_ligand_representation_transition_story(self):
        """Test ligand representation transition story."""
        result = ligand_representation_transition_story(self.test_url)
        
        assert "snapshots" in result
        assert len(result["snapshots"]) == 3  # sticks → surface → hybrid
        assert result["metadata"]["title"] == "Ligand Representation Transition"
        
    def test_protein_ligand_surface_story(self):
        """Test protein-ligand surface interaction story."""
        result = protein_ligand_surface_story(self.test_url)
        
        assert "snapshots" in result
        assert len(result["snapshots"]) == 4  # Multiple interaction stages
        assert result["metadata"]["title"] == "Protein-Ligand Surface Interaction"
        
    def test_hybrid_representation_showcase_story(self):
        """Test hybrid representation showcase story."""
        result = hybrid_representation_showcase_story(self.test_url)
        
        assert "snapshots" in result
        assert len(result["snapshots"]) == 6  # Different hybrid representations
        assert result["metadata"]["title"] == "Hybrid Representation Showcase"
        
    def test_story_with_custom_durations(self):
        """Test story with custom timing parameters."""
        configs = [
            {
                "url": self.test_url,
                "viz_type": "protein_cartoon", 
                "color": "blue",
                "title": "State 1",
                "duration": 3000,
                "transition_duration": 1500
            },
            {
                "url": self.test_url,
                "viz_type": "ligand_view",
                "color": "red", 
                "title": "State 2",
                "duration": 2500,
                "transition_duration": 500
            }
        ]
        
        result = create_multi_state_story(configs, "Timing Test")
        
        # Check that timing metadata is preserved
        assert result["snapshots"][0]["metadata"]["linger_duration_ms"] == 3000
        assert result["snapshots"][0]["metadata"]["transition_duration_ms"] == 1500
        assert result["snapshots"][1]["metadata"]["linger_duration_ms"] == 2500
        assert result["snapshots"][1]["metadata"]["transition_duration_ms"] == 500
        
    def test_invalid_viz_type(self):
        """Test that invalid visualization types raise errors."""
        configs = [
            {"url": self.test_url, "viz_type": "invalid_type", "color": "blue"}
        ]
        
        with pytest.raises(ValueError, match="Invalid viz_type"):
            create_multi_state_story(configs)
            
    def test_empty_configs(self):
        """Test that empty configurations raise errors."""
        with pytest.raises(ValueError, match="state_configs cannot be empty"):
            create_multi_state_story([])
            
    def test_surface_story_parameters(self):
        """Test surface-specific parameters in stories."""
        configs = [
            {
                "url": self.test_url,
                "viz_type": "multi_component",
                "color": "#e19039",
                "include_protein_surface": True,
                "protein_surface_opacity": 0.3,
                "include_ligand_surface": True,
                "ligand_surface_opacity": 0.6,
                "title": "Multi with Surfaces"
            }
        ]
        
        result = create_multi_state_story(configs, "Surface Parameters Test")
        
        assert len(result["snapshots"]) == 1
        assert result["snapshots"][0]["metadata"]["title"] == "Multi with Surfaces"