"""
Tests for surface visualization functions based on examples from the original code.
"""
import pytest
from mvs_story_generator.visualization import (
    protein_surface, ligand_surface, protein_surface_with_cartoon, ligand_surface_with_sticks
)


class TestSurfaceVisualization:
    """Test surface visualization functions."""
    
    def setup_method(self):
        """Set up test fixtures."""
        self.test_url = "https://www.ebi.ac.uk/pdbe/entry-files/1atp.bcif"
    
    def test_protein_surface_basic(self):
        """Test basic protein surface visualization."""
        result = protein_surface(
            self.test_url,
            "Test Protein Surface",
            color="#87CEEB",
            opacity=0.7
        )
        
        assert "metadata" in result
        assert "root" in result
        assert result["metadata"]["title"] == "Test Protein Surface"
        assert result["root"]["kind"] == "root"
        
    def test_protein_surface_with_probe_radius(self):
        """Test protein surface with custom probe radius."""
        result = protein_surface(
            self.test_url,
            "Test Protein Surface Custom Probe",
            color="#D0D0D0",
            opacity=0.5,
            probe_radius=2.0
        )
        
        assert "metadata" in result
        assert result["metadata"]["title"] == "Test Protein Surface Custom Probe"
        
    def test_ligand_surface_basic(self):
        """Test basic ligand surface visualization."""
        result = ligand_surface(
            self.test_url,
            "Test Ligand Surface", 
            color="#90EE90",
            opacity=0.6
        )
        
        assert "metadata" in result
        assert result["metadata"]["title"] == "Test Ligand Surface"
        
    def test_protein_surface_with_cartoon(self):
        """Test hybrid protein visualization with cartoon and surface."""
        result = protein_surface_with_cartoon(
            self.test_url,
            "Test Hybrid Protein",
            cartoon_color="#4577B2",
            surface_color="#D0D0D0",
            surface_opacity=0.4
        )
        
        assert "metadata" in result
        assert result["metadata"]["title"] == "Test Hybrid Protein"
        
    def test_ligand_surface_with_sticks(self):
        """Test hybrid ligand visualization with sticks and surface."""
        result = ligand_surface_with_sticks(
            self.test_url,
            "Test Hybrid Ligand",
            sticks_color="#FF6B6B", 
            surface_color="#90EE90",
            surface_opacity=0.5
        )
        
        assert "metadata" in result
        assert result["metadata"]["title"] == "Test Hybrid Ligand"
        
    def test_surface_with_camera_angles(self):
        """Test surface visualizations with different camera angles."""
        camera_angles = ["up", "right", "flip_horizontal"]
        
        for angle in camera_angles:
            result = protein_surface(
                self.test_url,
                f"Surface {angle}",
                camera_angle=angle
            )
            assert "metadata" in result
            assert result["metadata"]["title"] == f"Surface {angle}"
            
    def test_surface_with_labels(self):
        """Test surface visualization with labels."""
        labels = [
            {"text": "Surface Cavity", "selector": "polymer"},
            {"text": "Binding Site", "selector": "ligand"}
        ]
        
        result = protein_surface_with_cartoon(
            self.test_url,
            "Surface with Labels",
            labels=labels
        )
        
        assert "metadata" in result
        assert result["metadata"]["title"] == "Surface with Labels"
        
    def test_opacity_ranges(self):
        """Test different opacity values."""
        opacities = [0.1, 0.3, 0.5, 0.7, 0.9]
        
        for opacity in opacities:
            result = protein_surface(
                self.test_url,
                f"Opacity {opacity}",
                opacity=opacity
            )
            assert "metadata" in result
            
    def test_probe_radius_ranges(self):
        """Test different probe radius values."""
        probe_radii = [0.5, 1.0, 1.4, 2.0, 3.0]
        
        for radius in probe_radii:
            result = ligand_surface(
                self.test_url,
                f"Probe {radius}",
                probe_radius=radius
            )
            assert "metadata" in result