"""
Tests for validation functions based on the original code.
"""
import pytest
from mvs_story_generator.core.validation import _validate_and_normalize


class TestValidation:
    """Test validation functions."""
    
    def test_color_validation_hex(self):
        """Test hex color validation."""
        # Valid hex colors
        assert _validate_and_normalize("#ff0000", "color") == "#ff0000"
        assert _validate_and_normalize("ff0000", "color") == "#ff0000"
        assert _validate_and_normalize("#AABBCC", "color") == "#aabbcc"
        
    def test_color_validation_names(self):
        """Test color name validation."""
        # Valid color names
        assert _validate_and_normalize("red", "color") == "#ff0000"
        assert _validate_and_normalize("BLUE", "color") == "#0000ff"
        assert _validate_and_normalize(" green ", "color") == "#008000"
        assert _validate_and_normalize("protein_orange", "color") == "#e19039"
        
    def test_color_validation_invalid(self):
        """Test invalid color validation."""
        with pytest.raises(ValueError, match="Invalid color"):
            _validate_and_normalize("not_a_color", "color")
            
        with pytest.raises(ValueError, match="Invalid color"):
            _validate_and_normalize("#gggggg", "color")
            
        with pytest.raises(ValueError, match="Invalid color"):
            _validate_and_normalize("#ff00", "color")
            
    def test_camera_angle_validation(self):
        """Test camera angle validation."""
        # Valid angles
        valid_angles = ["up", "down", "left", "right", "flip_horizontal", "flip_vertical"]
        for angle in valid_angles:
            assert _validate_and_normalize(angle, "camera_angle") == angle
            
        # Case insensitive
        assert _validate_and_normalize("UP", "camera_angle") == "up"
        assert _validate_and_normalize(" left ", "camera_angle") == "left"
        
    def test_camera_angle_validation_invalid(self):
        """Test invalid camera angle validation."""
        with pytest.raises(ValueError, match="Invalid camera angle"):
            _validate_and_normalize("invalid_angle", "camera_angle")
            
    def test_text_validation(self):
        """Test text validation."""
        # Valid text
        assert _validate_and_normalize("Valid text", "text") == "Valid text"
        assert _validate_and_normalize("  trimmed  ", "text") == "trimmed"
        
        # Text with special characters
        valid_text = "Text with\nnewline and\ttab"
        assert _validate_and_normalize(valid_text, "text") == valid_text
        
    def test_text_validation_invalid(self):
        """Test invalid text validation."""
        # Empty text
        with pytest.raises(ValueError, match="Text must be 1-200 characters"):
            _validate_and_normalize("", "text")
            
        with pytest.raises(ValueError, match="Text must be 1-200 characters"):
            _validate_and_normalize("   ", "text")
            
        # Too long text
        long_text = "x" * 201
        with pytest.raises(ValueError, match="Text must be 1-200 characters"):
            _validate_and_normalize(long_text, "text")
            
    def test_selector_validation_string(self):
        """Test string selector validation."""
        valid_selectors = ["all", "polymer", "protein", "nucleic", "branched", "ligand", "ion", "water"]
        
        for selector in valid_selectors:
            assert _validate_and_normalize(selector, "selector") == selector
            assert _validate_and_normalize(selector.upper(), "selector") == selector
            
    def test_selector_validation_dict(self):
        """Test dictionary selector validation."""
        valid_dict = {"label_asym_id": "A", "label_seq_id": 72}
        result = _validate_and_normalize(valid_dict, "selector")
        assert result == valid_dict
        
        # Test with multiple valid keys
        complex_dict = {
            "label_entity_id": "1",
            "auth_asym_id": "A", 
            "beg_label_seq_id": 10,
            "end_label_seq_id": 20
        }
        result = _validate_and_normalize(complex_dict, "selector")
        assert result == complex_dict
        
    def test_selector_validation_invalid(self):
        """Test invalid selector validation."""
        # Invalid string selector
        with pytest.raises(ValueError, match="Invalid static selector"):
            _validate_and_normalize("invalid_selector", "selector")
            
        # Invalid dict keys
        with pytest.raises(ValueError, match="Invalid ComponentExpression keys"):
            _validate_and_normalize({"invalid_key": "value"}, "selector")
            
        # Wrong type
        with pytest.raises(ValueError, match="Selector must be string or dictionary"):
            _validate_and_normalize(123, "selector")
            
    def test_opacity_validation(self):
        """Test opacity validation."""
        # Valid opacities
        assert _validate_and_normalize(0.5, "opacity") == 0.5
        assert _validate_and_normalize("0.7", "opacity") == 0.7
        assert _validate_and_normalize(0, "opacity") == 0.0
        assert _validate_and_normalize(1, "opacity") == 1.0
        
    def test_opacity_validation_invalid(self):
        """Test invalid opacity validation."""
        with pytest.raises(ValueError, match="Opacity must be between 0.0 and 1.0"):
            _validate_and_normalize(-0.1, "opacity")
            
        with pytest.raises(ValueError, match="Opacity must be between 0.0 and 1.0"):
            _validate_and_normalize(1.1, "opacity")
            
        with pytest.raises(ValueError, match="Invalid opacity"):
            _validate_and_normalize("not_a_number", "opacity")
            
    def test_probe_radius_validation(self):
        """Test probe radius validation."""
        # Valid probe radii
        assert _validate_and_normalize(1.4, "probe_radius") == 1.4
        assert _validate_and_normalize("2.0", "probe_radius") == 2.0
        assert _validate_and_normalize(0.1, "probe_radius") == 0.1
        assert _validate_and_normalize(5.0, "probe_radius") == 5.0
        
    def test_probe_radius_validation_invalid(self):
        """Test invalid probe radius validation."""
        with pytest.raises(ValueError, match="Probe radius must be between 0.1 and 5.0"):
            _validate_and_normalize(0.05, "probe_radius")
            
        with pytest.raises(ValueError, match="Probe radius must be between 0.1 and 5.0"):
            _validate_and_normalize(6.0, "probe_radius")
            
        with pytest.raises(ValueError, match="Invalid probe radius"):
            _validate_and_normalize("not_a_number", "probe_radius")
            
    def test_representation_type_validation(self):
        """Test representation type validation."""
        valid_types = ["cartoon", "ball_and_stick", "surface", "spacefill", "ribbon", "tube", "line"]
        
        for repr_type in valid_types:
            assert _validate_and_normalize(repr_type, "representation_type") == repr_type
            assert _validate_and_normalize(repr_type.upper(), "representation_type") == repr_type
            
    def test_representation_type_validation_invalid(self):
        """Test invalid representation type validation."""
        with pytest.raises(ValueError, match="Invalid representation type"):
            _validate_and_normalize("invalid_type", "representation_type")
            
    def test_none_values(self):
        """Test that None values are handled correctly."""
        for validator_type in ["color", "camera_angle", "text", "selector", "opacity", "probe_radius", "representation_type"]:
            result = _validate_and_normalize(None, validator_type)
            assert result is None