"""
Basic visualization functions for MVS Story Generator.
"""
from typing import Dict, Any, Optional, List, Tuple
from ..core.types import LabelConfig
from ..core.constants import DEFAULT_COLORS
from ..core.builders import _build_mvsj_base, _build_structure_chain, _build_representation, _build_focus, _build_label
from ..core.components import _consolidate_components, _add_to_component_group


def _build_basic_visualization(url: str, title: str, components_config: List[Dict],
                              scientific_annotation: Optional[str] = None) -> Dict[str, Any]:
    """Build basic single-state visualization with component configurations and optional scientific annotation."""
    mvsj = _build_mvsj_base(title)
    chain = _build_structure_chain(url)
    structure_node = chain["children"][0]["children"][0]
      
    # Process component configurations
    component_features = {}
      
    for config in components_config:
        selector = config["selector"]
           
        # Build representation
        repr_node = _build_representation(
            config.get("type", "cartoon"),
            config.get("color"),
            config.get("opacity")
        )
        _add_to_component_group(component_features, selector, repr_node)
           
        # Add focus if specified
        if config.get("focus"):
            focus_node = _build_focus(
                config.get("camera_angle"),
                config.get("radius_factor"),
                config.get("interactions", False)
            )
            _add_to_component_group(component_features, selector, focus_node)
           
        # Add labels if specified
        if config.get("labels"):
            for label_config in config["labels"]:
                label_node = _build_label(label_config["text"])
                _add_to_component_group(component_features, label_config["selector"], label_node)
      
    # Add scientific annotation as a global label if provided
    if scientific_annotation:
        annotation_label = _build_label(scientific_annotation)
        _add_to_component_group(component_features, "all", annotation_label)
      
    # Consolidate and add components
    components = _consolidate_components(component_features)
    structure_node["children"].extend(components)
    mvsj["root"]["children"].append(chain)
    return mvsj


def protein_cartoon(url: str, title: Optional[str] = None, color: Optional[str] = None,
                   camera_angle: Optional[str] = None, labels: Optional[List[LabelConfig]] = None,
                   include_surface: bool = False, surface_color: Optional[str] = None,
                   surface_opacity: Optional[float] = None, probe_radius: Optional[float] = None) -> Dict[str, Any]:
    """Generate protein cartoon visualization with optional surface representation."""
    if include_surface:
        # Import here to avoid circular import
        from .surface import protein_surface_with_cartoon
        return protein_surface_with_cartoon(
            url=url, title=title, cartoon_color=color,
            surface_color=surface_color, surface_opacity=surface_opacity,
            probe_radius=probe_radius, camera_angle=camera_angle, labels=labels
        )
    else:
        # Original cartoon-only behavior
        config = [{
            "selector": "polymer",
            "type": "cartoon",
            "color": color or DEFAULT_COLORS["protein"],
            "focus": True,
            "camera_angle": camera_angle,
            "radius_factor": 1.5,
            "labels": labels or []
        }]
        return _build_basic_visualization(url, title or "Protein Cartoon View", config)


def ligand_view(url: str, title: Optional[str] = None, color: Optional[str] = None,
               camera_angle: Optional[str] = None, labels: Optional[List[LabelConfig]] = None,
               include_surface: bool = False, surface_color: Optional[str] = None,
               surface_opacity: Optional[float] = None, probe_radius: Optional[float] = None) -> Dict[str, Any]:
    """Generate ligand focus visualization with optional surface representation."""
    if include_surface:
        # Import here to avoid circular import
        from .surface import ligand_surface_with_sticks
        return ligand_surface_with_sticks(
            url=url, title=title, sticks_color=color,
            surface_color=surface_color, surface_opacity=surface_opacity,
            probe_radius=probe_radius, camera_angle=camera_angle, labels=labels
        )
    else:
        # Original ball_and_stick-only behavior
        config = [{
            "selector": "ligand",
            "type": "ball_and_stick",
            "color": color or DEFAULT_COLORS["ligand"],
            "focus": True,
            "camera_angle": camera_angle,
            "radius_factor": 2.0,
            "labels": labels or []
        }]
        return _build_basic_visualization(url, title or "Ligand Focus View", config)


def multi_component_colored_view(url: str, title: Optional[str] = None,
                                protein_color: Optional[str] = None,
                                nucleic_color: Optional[str] = None,
                                ligand_color: Optional[str] = None,
                                highlight_residues: Optional[List[Tuple[str, int]]] = None,
                                highlight_color: Optional[str] = None,
                                camera_angle: Optional[str] = None,
                                labels: Optional[List[LabelConfig]] = None,
                                include_protein_surface: bool = False,
                                protein_surface_color: Optional[str] = None,
                                protein_surface_opacity: Optional[float] = None,
                                include_ligand_surface: bool = False,
                                ligand_surface_color: Optional[str] = None,
                                ligand_surface_opacity: Optional[float] = None,
                                probe_radius: Optional[float] = None) -> Dict[str, Any]:
    """Generate multi-component colored visualization with optional surface representations."""
    config = [
        {
            "selector": "polymer",
            "type": "cartoon",
            "color": protein_color or DEFAULT_COLORS["protein"]
        },
        {
            "selector": "nucleic",
            "type": "cartoon",
            "color": nucleic_color or DEFAULT_COLORS["nucleic"]
        },
        {
            "selector": "ligand",
            "type": "ball_and_stick",
            "color": ligand_color or DEFAULT_COLORS["ligand"],
            "focus": True,
            "camera_angle": camera_angle,
            "radius_factor": 1.8,
            "labels": labels or []
        }
    ]
      
    # Add protein surface if requested
    if include_protein_surface:
        config.append({
            "selector": "polymer",
            "type": "surface",
            "color": protein_surface_color or "#D0D0D0",
            "opacity": protein_surface_opacity or 0.3,
            "probe_radius": probe_radius
        })
      
    # Add ligand surface if requested
    if include_ligand_surface:
        config.append({
            "selector": "ligand",
            "type": "surface",
            "color": ligand_surface_color or "#90EE90",
            "opacity": ligand_surface_opacity or 0.5,
            "probe_radius": probe_radius
        })
      
    # Add highlighted residues
    if highlight_residues:
        for chain_id, residue_number in highlight_residues:
            config.append({
                "selector": {"label_asym_id": chain_id, "label_seq_id": residue_number},
                "type": "ball_and_stick",
                "color": highlight_color or DEFAULT_COLORS["highlight"],
                "focus": True,
                "camera_angle": camera_angle,
                "radius_factor": 2.0
            })
      
    return _build_basic_visualization(url, title or "Multi-Component Colored View", config)


def multi_component_ligand_focus(url: str, title: Optional[str] = None,
                                protein_color: Optional[str] = None,
                                ligand_color: Optional[str] = None,
                                camera_angle: Optional[str] = None,
                                labels: Optional[List[LabelConfig]] = None,
                                include_protein_surface: bool = False,
                                protein_surface_color: Optional[str] = None,
                                protein_surface_opacity: Optional[float] = None,
                                include_ligand_surface: bool = False,
                                ligand_surface_color: Optional[str] = None,
                                ligand_surface_opacity: Optional[float] = None,
                                probe_radius: Optional[float] = None) -> Dict[str, Any]:
    """Generate ligand focus with protein context and optional surface representations."""
    config = [
        {
            "selector": "polymer",
            "type": "cartoon",
            "color": protein_color or "#B0B0B0",
            "opacity": 0.5
        },
        {
            "selector": "ligand",
            "type": "ball_and_stick",
            "color": ligand_color or "#FF6B6B",
            "focus": True,
            "camera_angle": camera_angle,
            "radius_factor": 2.0,
            "labels": labels or []
        }
    ]
      
    # Add protein surface if requested
    if include_protein_surface:
        config.append({
            "selector": "polymer",
            "type": "surface",
            "color": protein_surface_color or "#D0D0D0",
            "opacity": protein_surface_opacity or 0.2,
            "probe_radius": probe_radius
        })
      
    # Add ligand surface if requested
    if include_ligand_surface:
        config.append({
            "selector": "ligand",
            "type": "surface",
            "color": ligand_surface_color or "#90EE90",
            "opacity": ligand_surface_opacity or 0.5,
            "probe_radius": probe_radius
        })
      
    return _build_basic_visualization(url, title or "Ligand Focus with Protein Context", config)