"""
Surface visualization functions for MVS Story Generator.
"""
from typing import Dict, Any, Optional, List
from ..core.types import LabelConfig
from ..core.constants import DEFAULT_COLORS
from ..core.builders import _build_mvsj_base, _build_structure_chain, _build_representation, _build_focus, _build_label
from ..core.components import _consolidate_components, _add_to_component_group
from .basic import _build_basic_visualization


def protein_surface(url: str, title: Optional[str] = None, color: Optional[str] = None,
                   opacity: Optional[float] = None, probe_radius: Optional[float] = None,
                   camera_angle: Optional[str] = None, labels: Optional[List[LabelConfig]] = None) -> Dict[str, Any]:
    """Generate standalone protein surface visualization."""
    config = [{
        "selector": "polymer",
        "type": "surface",
        "color": color or "#D0D0D0",
        "opacity": opacity or 0.7,
        "probe_radius": probe_radius,
        "focus": True,
        "camera_angle": camera_angle,
        "radius_factor": 1.5,
        "labels": labels or []
    }]
    return _build_basic_visualization(url, title or "Protein Surface View", config)


def ligand_surface(url: str, title: Optional[str] = None, color: Optional[str] = None,
                  opacity: Optional[float] = None, probe_radius: Optional[float] = None,
                  camera_angle: Optional[str] = None, labels: Optional[List[LabelConfig]] = None) -> Dict[str, Any]:
    """Generate standalone ligand surface visualization."""
    config = [{
        "selector": "ligand",
        "type": "surface",
        "color": color or "#90EE90",
        "opacity": opacity or 0.7,
        "probe_radius": probe_radius,
        "focus": True,
        "camera_angle": camera_angle,
        "radius_factor": 2.0,
        "labels": labels or []
    }]
    return _build_basic_visualization(url, title or "Ligand Surface View", config)


def protein_surface_with_cartoon(url: str, title: Optional[str] = None,
                                cartoon_color: Optional[str] = None,
                                surface_color: Optional[str] = None,
                                surface_opacity: Optional[float] = None,
                                probe_radius: Optional[float] = None,
                                camera_angle: Optional[str] = None,
                                labels: Optional[List[LabelConfig]] = None) -> Dict[str, Any]:
    """Generate hybrid protein visualization with both cartoon and surface representations."""
    # Build multiple representations for the same selector
    mvsj = _build_mvsj_base(title or "Protein Surface with Cartoon")
    chain = _build_structure_chain(url)
    structure_node = chain["children"][0]["children"][0]
      
    # Create component with multiple representations
    component_features = {}
    selector = "polymer"
      
    # Add cartoon representation
    cartoon_repr = _build_representation(
        "cartoon",
        cartoon_color or DEFAULT_COLORS["protein"]
    )
    _add_to_component_group(component_features, selector, cartoon_repr)
      
    # Add surface representation
    surface_repr = _build_representation(
        "surface",
        surface_color or "#D0D0D0",
        surface_opacity or 0.3,
        probe_radius
    )
    _add_to_component_group(component_features, selector, surface_repr)
      
    # Add focus
    focus_node = _build_focus(camera_angle, 1.5)
    _add_to_component_group(component_features, selector, focus_node)
      
    # Add labels if specified
    if labels:
        for label_config in labels:
            label_node = _build_label(label_config["text"])
            _add_to_component_group(component_features, label_config["selector"], label_node)
      
    # Consolidate and add components
    components = _consolidate_components(component_features)
    structure_node["children"].extend(components)
    mvsj["root"]["children"].append(chain)
    return mvsj


def ligand_surface_with_sticks(url: str, title: Optional[str] = None,
                              sticks_color: Optional[str] = None,
                              surface_color: Optional[str] = None,
                              surface_opacity: Optional[float] = None,
                              probe_radius: Optional[float] = None,
                              camera_angle: Optional[str] = None,
                              labels: Optional[List[LabelConfig]] = None) -> Dict[str, Any]:
    """Generate hybrid ligand visualization with both ball_and_stick and surface representations."""
    # Build multiple representations for the same selector
    mvsj = _build_mvsj_base(title or "Ligand Surface with Ball and Stick")
    chain = _build_structure_chain(url)
    structure_node = chain["children"][0]["children"][0]
      
    # Create component with multiple representations
    component_features = {}
    selector = "ligand"
      
    # Add ball_and_stick representation
    sticks_repr = _build_representation(
        "ball_and_stick",
        sticks_color or DEFAULT_COLORS["ligand"]
    )
    _add_to_component_group(component_features, selector, sticks_repr)
      
    # Add surface representation
    surface_repr = _build_representation(
        "surface",
        surface_color or "#90EE90",
        surface_opacity or 0.5,
        probe_radius
    )
    _add_to_component_group(component_features, selector, surface_repr)
      
    # Add focus
    focus_node = _build_focus(camera_angle, 2.0)
    _add_to_component_group(component_features, selector, focus_node)
      
    # Add labels if specified
    if labels:
        for label_config in labels:
            label_node = _build_label(label_config["text"])
            _add_to_component_group(component_features, label_config["selector"], label_node)
      
    # Consolidate and add components
    components = _consolidate_components(component_features)
    structure_node["children"].extend(components)
    mvsj["root"]["children"].append(chain)
    return mvsj